﻿if (GetLocale() == "zhTW") then
TBAGS_MYBAGS = "我的背包";
TBAGS_CLOSEBAGS = "關閉背包";
TBAGS_PACKBAGS = "整理背包";

elseif (GetLocale() == "zhCN") then
TBAGS_MYBAGS = "我的背包";
TBAGS_CLOSEBAGS = "关闭背包";
TBAGS_PACKBAGS = "整理背包";

else
TBAGS_MYBAGS = "MyBags";
TBAGS_CLOSEBAGS = "CloseBags";
TBAGS_PACKBAGS = "PackBags";
end

local addon, ns = ...
local cargBags = ns.cargBags



local m_Bags = cargBags:NewImplementation("m_Bags")	-- Let the magic begin!
m_Bags:RegisterBlizzard() -- register the frame for use with BLizzard's ToggleBag()-functions
--local m_Bags = cargBags:GetImplementation("m_Bags")

-- A highlight function styles the button if they match a certain condition
local function highlightFunction(button, match)
	button:SetAlpha(match and 1 or 0.1)
end

local f = {}
function m_Bags:OnInit()
	-- The filters control which items go into which container
	local INVERTED = -1 -- with inverted filters (using -1), everything goes into this bag when the filter returns false
	local onlyBags =		function(item) return item.bagID >= 0 and item.bagID <= 4 end
	local onlyBank =		function(item) return item.bagID == -1 or item.bagID >= 5 and item.bagID <= 11 end
	local onlyRareEpics =	function(item) return item.rarity and item.rarity > 3 end
	local onlyEpics =		function(item) return item.rarity and item.rarity > 3 end
	local hideJunk =		function(item) return not item.rarity or item.rarity > 0 end
	local hideEmpty =		function(item) return item.texture ~= nil end

	local MyContainer = m_Bags:GetContainerClass()
	
	-- Bagpack
	f.main = MyContainer:New("Main", {
			Columns = 11,
			Scale = 0.9,
			Bags = "backpack+bags",
			Movable = true,
	})
	f.main:SetFilter(onlyBags, true)
	f.main:SetPoint("BOTTOMRIGHT", -10*f.main.Settings.Scale, 150*f.main.Settings.Scale) -- bagpack position

	-- Bank frame and bank bags
	f.bank = MyContainer:New("Bank", {
			Columns = 16,
			Scale = f.main.Settings.Scale,
			Bags = "bankframe+bank",
	})
	f.bank:SetFilter(onlyBank, true) -- Take only items from the bank frame
	f.bank:SetPoint("BOTTOMRIGHT", f.main,"BOTTOMLEFT", -25*f.main.Settings.Scale, 0) -- bank frame position
	f.bank:Hide() -- Hide at the beginning
	

end

-- Bank frame toggling
function m_Bags:OnBankOpened()
	self:GetContainer("Bank"):Show()
end

function m_Bags:OnBankClosed()
	self:GetContainer("Bank"):Hide()
end

-- Class: ItemButton appearencence classification
local MyButton = m_Bags:GetItemButtonClass()
MyButton:Scaffold("Default")
function MyButton:OnUpdate(item)
	-- color the border based on bag type
	local bagType = (select(2, GetContainerNumFreeSlots(self.bagID)));
	-- ammo / soulshards
	if(bagType and (bagType > 0 and bagType < 8)) then
		self.Border:SetVertexColor(0.85, 0.85, 0.35, 1);
	-- profession bags
	elseif(bagType and bagType > 4) then
		self.Border:SetVertexColor(0.1, 0.65, 0.1, 1);
	-- normal bags
	else
		self.Border:SetVertexColor(.7, .7, .7, .9);
	end
end

--	Class: BagButton is the template for all buttons on the BagBar
local BagButton = m_Bags:GetClass("BagButton", true, "BagButton")
-- We color the CheckedTexture golden, not bright yellow
function BagButton:OnCreate()
	self:GetCheckedTexture():SetVertexColor(0.3, 0.9, 0.9, 0.5)
end

-- Class: Container (Serves as a base for all containers/bags)
-- Fetch our container class that serves as a basis for all our containers/bags

local UpdateDimensions = function(self)
	local width, height = self:LayoutButtons("grid", self.Settings.Columns, 3, 6, -5)
	--local width, height = self:GetWidth(), self:GetHeight()
	local margin = 45			-- Normal margin space for infobar
	if self.BagBar and self.BagBar:IsShown() then
		margin = margin + 40	-- Bag button space
	end
	self:SetHeight(height + margin)
end

local MyContainer = m_Bags:GetContainerClass()
function MyContainer:OnContentsChanged()
	-- sort our buttons based on the slotID
	self:SortButtons("bagSlot")
	-- Order the buttons in a layout, ("grid", columns, spacing, xOffset, yOffset) or ("circle", radius (optional), xOffset, yOffset)
	local width, height = self:LayoutButtons("grid", self.Settings.Columns, 3, 6, -5)
	self:SetSize(width + 12, height + 12)
	if (self.UpdateDimensions) then self:UpdateDimensions() end -- Update the bag's height
end

-- OnCreate is called every time a new container is created 
function MyContainer:OnCreate(name, settings)
	settings = settings or {}
    self.Settings = settings
	self.UpdateDimensions = UpdateDimensions
	
	self:EnableMouse(true)

	self:SetBackdrop{
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 10, edgeSize = 8,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	}
	self:SetBackdropColor(0,0,0,0.5)
	self:SetBackdropBorderColor(0,0,0,1)

	self:SetParent(settings.Parent or m_Bags)
	self:SetFrameStrata("HIGH")

	if(settings.Movable) then
		self:SetMovable(true)
		self:RegisterForClicks("LeftButton", "RightButton");
	    self:SetScript("OnMouseDown", function()
			if IsAltKeyDown() then
				self:ClearAllPoints() 
				self:StartMoving()
			end
	    end)
		self:SetScript("OnMouseUp",  self.StopMovingOrSizing)
	end

	settings.Columns = settings.Columns or 10
	self:SetScale(settings.Scale or 1)
	
	-- Creating infoFrame which serves as a basic bar for information and extra buttons
	local infoFrame = CreateFrame("Button", nil, self)
	infoFrame:SetPoint("BOTTOMLEFT", 150, 0)
	infoFrame:SetWidth(100)
	infoFrame:SetHeight(25)

	-- Plugin: TagDisplay
	-- This one shows currencies, ammo and - most important - money!
	local tagDisplay = self:SpawnPlugin("TagDisplay", "[money]", infoFrame)
	tagDisplay:SetFontObject("NumberFontNormal")
	tagDisplay:SetFont("Fonts\\ARIALN.ttf", 14)
	tagDisplay:SetPoint("CENTER", infoFrame,"RIGHT",0,0)
		
	-- Plugin: BagBar
	local bagBar = self:SpawnPlugin("BagBar", settings.Bags)
	bagBar:SetSize(bagBar:LayoutButtons("grid", 7))
	bagBar:SetScale(0.75)
	bagBar.highlightFunction = highlightFunction -- from above, optional, used when hovering over bag buttons
	bagBar.isGlobal = true -- This would make the hover-effect apply to all containers instead of the current one
	bagBar:Hide()
	self.BagBar = bagBar
		
	-- positioning our BagBar
	bagBar:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 10, 50)
		

		
	-- creating button for toggling BagBar on and off
	self:UpdateDimensions()
	local bagToggle = CreateFrame("CheckButton", nil, self)
	bagToggle:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
	bagToggle:SetWidth(72)
	bagToggle:SetHeight(20)
	bagToggle:SetPoint("BOTTOMLEFT",0,2)
	bagToggle:RegisterForClicks("LeftButtonUp")
	bagToggle:SetScript("OnClick", function()
		if(self.BagBar:IsShown()) then
			self.BagBar:Hide()
		else
			self.BagBar:Show()
		end
			self:UpdateDimensions()
	end)
	local bagToggleText = bagToggle:CreateFontString(nil, "OVERLAY")
	bagToggleText:SetPoint("CENTER", bagToggle)
	bagToggleText:SetFont("Fonts\\ARIALN.ttf", 15 , "THINOUTLINE")
	bagToggleText:SetText(TBAGS_MYBAGS)
		
	-- Jpack Button --
	local JpackButton = CreateFrame("Button", nil, self)
	JpackButton:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
	JpackButton:SetWidth(72)
	JpackButton:SetHeight(20)
	JpackButton:SetPoint("BOTTOMRIGHT",0,2)
	JpackButton:SetScript("OnClick", function() JPack:Pack() end)
	local JpackButtonText = JpackButton:CreateFontString(nil, "OVERLAY")
	JpackButtonText:SetPoint("CENTER", JpackButton)
	JpackButtonText:SetFont("Fonts\\ARIALN.ttf", 15, "THINOUTLINE")
	JpackButtonText:SetText(TBAGS_PACKBAGS)
	
	local closebutton = CreateFrame("CheckButton", nil, self)
	closebutton:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
	closebutton:SetWidth(72)
	closebutton:SetHeight(20)
	closebutton:SetPoint("BOTTOMLEFT",75,2)
	closebutton:RegisterForClicks("LeftButtonUp")
	closebutton:SetScript("OnClick", function(self) 
		if m_Bags:AtBank() and self.name == "Bank" then 
			CloseBankFrame() 
		else 
			CloseAllBags() 
		end 
	end)
	local closebuttonText = closebutton:CreateFontString(nil, "OVERLAY")
	closebuttonText:SetPoint("CENTER", closebutton)
	closebuttonText:SetFont("Fonts\\ARIALN.ttf", 15 , "THINOUTLINE")
	closebuttonText:SetText(TBAGS_CLOSEBAGS)	
--登录
local LoadedFrame = CreateFrame("FRAME", "LoadedFrame");
LoadedFrame:RegisterEvent("ADDON_LOADED");
LoadedFrame:RegisterEvent("PLAYER_LOGOUT");
function LoadedFrame:OnEvent(event)
   if event == "ADDON_LOADED" then
      CurrInfos()      
   end
end
function CurrInfos()
   UpdateJusticeInfo()
   UpdateHonorInfo()
   UpdateConquestInfo()
   UpdateValorInfo()
   UpdateIronpawInfo()
end
LoadedFrame:SetScript("OnEvent", LoadedFrame.OnEvent)
	
	--荣誉点
self:CreateFontString("BagHonorText","OVERLAY","NumberFontNormalSmall");
BagHonorText:SetPoint("BOTTOMLEFT" ,100, 22);
BagHonorText:SetTextColor(1,1,1,1);
BagHonorText:SetFont("Fonts\\ARIALN.ttf", 14, "THINOUTLINE")
function GetHonor()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(392)
   return CurrInfo
end

function UpdateHonorInfo()
   BagHonorText:SetText(GetHonor())
end
UpdateHonorInfo()

local HonorFrameIcon = CreateFrame("frame", "HonorFrameIcon", self)

function HonorIcon()
   local EnglishFaction, localizedFaction = UnitFactionGroup("player")
   if EnglishFaction == "Horde" then
      HonorFrameIcon:SetBackdrop({
            bgFile = [[Interface\ICONS\PVPCurrency-Honor-Horde]],
			edgeFile = 'Interface\\Buttons\\WHITE8x8', edgeSize = 2, insets = {left = 0, right = 0, top = 0, bottom = 0}
      })
      HonorFrameIcon:SetBackdropColor(1, 1, 1, 1)
      HonorFrameIcon:SetBackdropBorderColor(0, 0, 0, 1)
   else
      HonorFrameIcon:SetBackdrop({
            bgFile = [[Interface\ICONS\PVPCurrency-Honor-Alliance]],
			edgeFile = 'Interface\\Buttons\\WHITE8x8', edgeSize = 2, insets = {left = 0, right = 0, top = 0, bottom = 0}
      })
	  HonorFrameIcon:SetBackdropColor(1, 1, 1, 1)
      HonorFrameIcon:SetBackdropBorderColor(0, 0, 0, 1)
   end
end

HonorIcon()
HonorFrameIcon:SetWidth(14)
HonorFrameIcon:SetHeight(14)
HonorFrameIcon:SetPoint("BOTTOMRIGHT",BagHonorText,"BOTTOMLEFT", -10, 0)
HonorFrameIcon:Show()




--正义点


local JusticeFrameIcon = CreateFrame("frame", "JusticeFrameIcon", self)
function JusticeIcon()

    JusticeFrameIcon:SetBackdrop({
            bgFile = [[Interface\ICONS\PVECurrency-Justice]],
			edgeFile = 'Interface\\Buttons\\WHITE8x8', edgeSize = 2, insets = {left = 0, right = 0, top = 0, bottom = 0}
     })
      JusticeFrameIcon:SetBackdropColor(1, 1, 1, 1)
      JusticeFrameIcon:SetBackdropBorderColor(0, 0, 0, 1)
end
JusticeIcon()
JusticeFrameIcon:SetWidth(14)
JusticeFrameIcon:SetHeight(14)
JusticeFrameIcon:SetPoint("BOTTOMLEFT",BagHonorText,"BOTTOMRIGHT" ,10, 0)
JusticeFrameIcon:Show()

self:CreateFontString("BagJusticeText","OVERLAY","NumberFontNormalSmall");
BagJusticeText:SetPoint("BOTTOMLEFT",JusticeFrameIcon,"BOTTOMRIGHT" ,10, 0);
BagJusticeText:SetTextColor(1,1,1,1);
BagJusticeText:SetFont("Fonts\\ARIALN.ttf", 14, "THINOUTLINE")
function GetJustice()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(395)
   return CurrInfo
end
function UpdateJusticeInfo()
   BagJusticeText:SetText(GetJustice())
end
UpdateJusticeInfo()

--征服点


local ConquestFrameIcon = CreateFrame("frame", "ConquestFrameIcon", self)
function ConquestIcon()
   local EnglishFaction, localizedFaction = UnitFactionGroup("player")
   if EnglishFaction == "Horde" then
      ConquestFrameIcon:SetBackdrop({
            bgFile = [[Interface\ICONS\PVPCurrency-Conquest-Horde]],
            edgeFile = 'Interface\\Buttons\\WHITE8x8', edgeSize = 2, insets = {left = 0, right = 0, top = 0, bottom = 0}
     })
      ConquestFrameIcon:SetBackdropColor(1, 1, 1, 1)
      ConquestFrameIcon:SetBackdropBorderColor(0, 0, 0, 1)
   else
      ConquestFrameIcon:SetBackdrop({
            bgFile = [[Interface\ICONS\PVPCurrency-Conquest-Alliance]],
            edgeFile = 'Interface\\Buttons\\WHITE8x8', edgeSize = 2, insets = {left = 0, right = 0, top = 0, bottom = 0}
     })
      ConquestFrameIcon:SetBackdropColor(1, 1, 1, 1)
      ConquestFrameIcon:SetBackdropBorderColor(0, 0, 0, 1)
   end
end
ConquestIcon()
ConquestFrameIcon:SetWidth(14)
ConquestFrameIcon:SetHeight(14)
ConquestFrameIcon:SetPoint("BOTTOMLEFT",BagJusticeText,"BOTTOMRIGHT" ,10, 0)
ConquestFrameIcon:Show()

local ConquestEventFrame = CreateFrame("FRAME", "ConquestEventFrame");
ConquestEventFrame:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
ConquestEventFrame:SetScript("OnEvent", CurrInfos);

self:CreateFontString("BagConquestText","OVERLAY","NumberFontNormalSmall");
BagConquestText:SetPoint("BOTTOMLEFT",ConquestFrameIcon,"BOTTOMRIGHT" ,10, 0);
BagConquestText:SetTextColor(1,1,1,1);
BagConquestText:SetFont("Fonts\\ARIALN.ttf", 14, "THINOUTLINE")
function GetConquest()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(390)
   return CurrInfo
end
function UpdateConquestInfo()
   BagConquestText:SetText(GetConquest())
end
UpdateConquestInfo()

--勇气点


local ValorFrameIcon = CreateFrame("frame", "ValorFrameIcon", self)
function ValorIcon()

    ValorFrameIcon:SetBackdrop({
            bgFile = [[Interface\ICONS\PVECurrency-Valor]],
			edgeFile = 'Interface\\Buttons\\WHITE8x8', edgeSize = 2, insets = {left = 0, right = 0, top = 0, bottom = 0}
     })
      ValorFrameIcon:SetBackdropColor(1, 1, 1, 1)
      ValorFrameIcon:SetBackdropBorderColor(0, 0, 0, 1)
end
ValorIcon()
ValorFrameIcon:SetWidth(14)
ValorFrameIcon:SetHeight(14)
ValorFrameIcon:SetPoint("BOTTOMLEFT",BagConquestText,"BOTTOMRIGHT" ,10, 0)
ValorFrameIcon:Show()

self:CreateFontString("BagValorText","OVERLAY","NumberFontNormalSmall");
BagValorText:SetPoint("BOTTOMLEFT",ValorFrameIcon,"BOTTOMRIGHT" ,10, 0);
BagValorText:SetTextColor(1,1,1,1);
BagValorText:SetFont("Fonts\\ARIALN.ttf", 14, "THINOUTLINE")
function GetValor()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(396)
   return CurrInfo
end
function UpdateValorInfo()
   BagValorText:SetText(GetValor())
end
UpdateValorInfo()


--铁掌徽记
local IronpawFrameIcon = CreateFrame("frame", "IronpawFrameIcon", self)
function IronpawIcon()
    IronpawFrameIcon:SetBackdrop({
            bgFile =  [[Interface\Icons\inv_relics_idolofferocity]],
			edgeFile = 'Interface\\Buttons\\WHITE8x8', edgeSize = 2, insets = {left = 0, right = 0, top = 0, bottom = 0}
     })
      IronpawFrameIcon:SetBackdropColor(1, 1, 1, 1)
      IronpawFrameIcon:SetBackdropBorderColor(0, 0, 0, 1)
end
IronpawIcon()
IronpawFrameIcon:SetWidth(14)
IronpawFrameIcon:SetHeight(14)
IronpawFrameIcon:SetPoint("BOTTOMLEFT",BagValorText,"BOTTOMRIGHT" ,10, 0)
IronpawFrameIcon:Show()

self:CreateFontString("BagIronpawText","OVERLAY","NumberFontNormalSmall");
BagIronpawText:SetPoint("BOTTOMLEFT",IronpawFrameIcon,"BOTTOMRIGHT" ,10, 0);
BagIronpawText:SetTextColor(1,1,1,1);
BagIronpawText:SetFont("Fonts\\ARIALN.ttf", 14, "THINOUTLINE")
function GetIronpaw()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(402)
   return CurrInfo
end
function UpdateIronpawInfo()
   BagIronpawText:SetText(GetIronpaw())
end
UpdateIronpawInfo()

--local _,_,_,_,_,_,texture = GetCurrencyListInfo(5)
--print(texture)
--自动换区域
self:HookScript("OnEvent",function(self,event,...)
	if event=="ZONE_CHANGED_NEW_AREA" and not WorldMapFrame:IsShown() or OpenAllBags() or openBag() or m_Bags:Show() then
		CurrInfos()
	end
end);
	
end

